<?php

namespace Drupal\commerce_stripe_sofort\Controller;

use Drupal;
use Drupal\commerce_payment\Entity\Payment;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\commerce_stripe_sofort\Event\CommerceStripeSofortEvent;
use Drupal\commerce_stripe_sofort\Event\CommerceStripeSofortEvents;
use Stripe\PaymentIntent;
use Stripe\Exception\ApiErrorException;
use Stripe\Exception\InvalidRequestException;
use Stripe\Stripe;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;


/**
 * Class StripeWebhookController.
 */
class StripeWebhookController extends ControllerBase {

  public function __construct() {
    $stripeSofortConfiguration = Drupal::config('commerce_payment.commerce_payment_gateway.commerce_sofort_stripe');
    if (!is_null($secret = $stripeSofortConfiguration->get('configuration.secret_key'))) {
      Stripe::setApiKey($secret);
    }
  }

  /**
   * Capture the payload.
   *
   * @return Symfony\Component\HttpFoundation\Response
   *   A simple string and 200 response.
   */
  public function capture(Request $request) {
    $response = new Response();
    $payload = $request->getContent();
    // $payload = $this->simulatedPayload();
    if (empty($payload)) {
      $message = 'The payload was empty.';
      Drupal::logger('commerce_stripe_sofort')->error($message);
      $response->setContent($message);
      return $response;
    }

    // Add the $payload to our defined queue.
    $updateOrder = $this->updateOrder(Json::decode($payload));
    if ($updateOrder instanceof Payment) {
      $response->setContent('Order updated');
    }
    else {
      $response->setContent('Order not updated');
    }
    return $response;
  }

  /**
   * Simple authorization using a token.
   *
   * @param string $token
   *    A random token only your webhook knows about.
   *
   * @return AccessResult
   *   AccessResult allowed or forbidden.
   */
  public function authorize() {
    return AccessResult::allowed();
  }

  /**
   * Check the Stripe PaymentIntent if the order is paid and update the order
   * status accordingly
   *
   * @param array $data
   *
   * @return void
   */
  private function updateOrder($data) {
    try {
      $paymentIntentId = $data['data']['object']['id'];
      $paymentIntent = PaymentIntent::retrieve($paymentIntentId);
      $orderId = $paymentIntent->metadata->order_id;

      //update the order it is true
      $order = Drupal::entityTypeManager()
        ->getStorage('commerce_order')
        ->load($orderId);
      if (!is_null($order)) {
        $payment = NULL;
        $remoteId = $order->getData('commerce_stripe_sofort_intent');
        $remoteSecret = $order->getData('commerce_stripe_sofort_secret');
        $payment = $this->findPayment($remoteId, $remoteSecret);

        if ($payment instanceof Payment) {
          if (!$order->isPaid() && $paymentIntent->status == PaymentIntent::STATUS_SUCCEEDED) {
            $workflow = $payment->getState()->getWorkflow();
            $transition = $workflow->getTransition('capture');
            $payment->getState()->applyTransition($transition);
            $payment->setCompletedTime(Drupal::time()->getRequestTime());
            $payment->save();
            $event = new CommerceStripeSofortEvent($payment);
            Drupal::entityTypeManager()->getStorage('commerce_log')
              ->generate($order, 'commerce_stripe_sofort.capture_success', [
                'remote_id' => $payment->getRemoteId(),
              ])->save();
            Drupal::service('event_dispatcher')
              ->dispatch(CommerceStripeSofortEvents::COMMERCE_STRIPE_SOFORT_PAYMENT_SUCCEEDED, $event);
          }
          elseif (in_array($paymentIntent->status, [
            PaymentIntent::STATUS_REQUIRES_PAYMENT_METHOD,
            PaymentIntent::STATUS_CANCELED,
            PaymentIntent::STATUS_REQUIRES_ACTION,
          ])) {
            $workflow = $payment->getState()->getWorkflow();
            $transition = $workflow->getTransition('void');
            $payment->getState()->applyTransition($transition);
            $payment->save();
            $event = new CommerceStripeSofortEvent($payment);
            Drupal::service('event_dispatcher')
              ->dispatch(CommerceStripeSofortEvents::COMMERCE_STRIPE_SOFORT_PAYMENT_FAILED, $event);

            Drupal::entityTypeManager()->getStorage('commerce_log')
              ->generate($order, 'commerce_stripe_sofort.capture_declined', [
                'remote_id' => $payment->getRemoteId(),
              ])->save();
          }
        }
        return $payment;
      }
    } catch (ApiErrorException  $e) {
      throw new InvalidRequestException('Unable to fetch Stripe payment intent');
      Drupal::logger('commerce_stripe_sofort')->notice($e->getMessage());
    }
  }

  /**
   * Finds the payment by remote_id and client_secret combination.
   *
   * @param string $remote_id
   *   Remote id.
   * @param string|bool $client_secret
   *   If not empty checks if client secret matches the one on payment intent.
   *
   * @return \Drupal\commerce_payment\Entity\Payment|null
   *   Either the found payment entity or null if nothing could be found.
   */
  public function findPayment($remote_id, $client_secret = FALSE) {
    /** @var \Drupal\commerce_payment\Entity\Payment[] $payments */
    $payments = Drupal::entityTypeManager()
      ->getStorage('commerce_payment')
      ->loadByProperties([
        'remote_id' => $remote_id,
      ]);
    if (empty($payments)) {
      return NULL;
    }
    $payment = reset($payments);
    if ($payment instanceof Payment && $client_secret) {
      $order = $payment->getOrder();
      if ($order instanceof Order && $client_secret != $order->getData('commerce_stripe_sofort_secret')) {
        // Payment doesn't match client secret.
        $payment = NULL;
      }
    }

    return $payment;
  }

}
